////////////////////////////////////////////////////////////////////////////////
//
// Fracter - A simple Mandelbrot Set viewer.
//
// Copyright (C) 2010 Sarah Eris Horsley Caffee
//
// This file is part of Fracter.
//
// Fracter is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
// EventHandler.h
// EventHandler class definition.  This is a virtual class that is meant to be
// an interface implemented by an application class.  It does, though, provide
// default implementations of it's methods so that unused events can be quietly
// ignored.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef EVENTHANDLER_H
#define EVENTHANDLER_H

// Use SDL 1.3 only!
#ifndef SDL_NO_COMPAT
#define SDL_NO_COMPAT
#endif

#include "SDL.h"

class EventHandler
   {
   public:

   EventHandler();
   virtual ~EventHandler() = 0;

   virtual void on_Event(SDL_Event * event);

   // SDL_WindowEvent subtypes
   virtual void on_Show();
   virtual void on_Hide();
   virtual void on_Expose();
   virtual void on_Move(SDL_WindowEvent window);
   virtual void on_Resize(SDL_WindowEvent window);
   virtual void on_Minimize();
   virtual void on_Maximize();
   virtual void on_Restore();
   virtual void on_MouseFocus();
   virtual void on_MouseBlur();
   virtual void on_KeyboardFocus();
   virtual void on_KeyboardBlur();
   // Note: on_Close() will call on_Quit() by default.
   virtual void on_Close();

   // Keyboard events
   virtual void on_KeyDown(SDL_KeyboardEvent key);
   virtual void on_KeyUp(SDL_KeyboardEvent key);
   virtual void on_TextInput(SDL_TextInputEvent text);
   virtual void on_TextEdit(SDL_TextEditingEvent edit);

   // Mouse events
   virtual void on_MouseMove(SDL_MouseMotionEvent motion);
   virtual void on_LButtonDown(SDL_MouseButtonEvent button);
   virtual void on_LButtonUp(SDL_MouseButtonEvent button);
   virtual void on_RButtonDown(SDL_MouseButtonEvent button);
   virtual void on_RButtonUp(SDL_MouseButtonEvent button);
   virtual void on_MButtonDown(SDL_MouseButtonEvent button);
   virtual void on_MButtonUp(SDL_MouseButtonEvent button);
   virtual void on_MouseWheel(SDL_MouseWheelEvent wheel);

   // Joystick events
   virtual void on_JoyAxis(SDL_JoyAxisEvent jaxis);
   virtual void on_JoyBall(SDL_JoyBallEvent jball);
   virtual void on_JoyHat(SDL_JoyHatEvent jhat);
   virtual void on_JoyButtonDown(SDL_JoyButtonEvent jbutton);
   virtual void on_JoyButtonUp(SDL_JoyButtonEvent jbutton);

   // Misc events
   virtual void on_Quit();
   virtual void on_User(SDL_UserEvent user);
   virtual void on_SysWMEvent(SDL_SysWMEvent syswm);

   // There's still a lot of new stuff not yet implemented here.

   };

#endif
